package SeMeet::Controller::Meets;
use Mojo::Base 'Mojolicious::Controller', -signatures;

use UUID::URandom qw(create_uuid_string);
use Mojo::JWT;

use constant AVATAR_SIZE          => 320;
use constant JITSI_TOKEN_LIFETIME => 3600 * 24;

sub create($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $exists = $c->schema->resultset('Meet')->count({
        name => $args->{name}
    });

    return $c->error(400, 'DUPLICTE_NAME') if $exists;

    my $guard  = $c->schema->txn_scope_guard;

        my $meet = $c->schema->resultset('Meet')->create({
            uuid     => create_uuid_string(),
            owner_id => $c->stash->{user}->id,
            name     => $args->{name},
        });

        $meet->add_to_meet_users({
            user_id      => $c->current_user->{id},
            is_moderator => 't',
        });

    $guard->commit;

    $c->trace(\'User %s create meet "%s" with id %d',
        $c->stash->{user}->username,
        $meet->name,
        $meet->id,
    );

    $c->render(
        status  => 201,
        openapi => { id => $meet->id },
    );
}

sub get($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;

    $c->render(openapi => $c->spec_filter({
        $meet->get_columns,
        groups => $meet->cached_groups,
        moderators  => $meet->cached_moderators,
        is_editable => ( $meet->owner_id == $c->stash->{user}->id),
    }, 'Meet'));
}

sub list($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meets = $c->schema->resultset('Meet')->search({
        deleted  => undef,
        owner_id => $c->stash->{user}->id,
# PODMINKA PRO UCASTNIKY
    },
    {
        order_by => 'name',
    }
    );

    my @meets = ();

    MEET:
    while ( my $meet = $meets->next()) {
        push @meets, $c->spec_filter({
            $meet->get_columns,
            groups => $meet->cached_groups,
            moderators => $meet->cached_moderators,
            is_editable => ( $meet->owner_id == $c->stash->{user}->id),
        }, 'Meet')
    }

    $c->render(
        openapi => {
            count   => scalar( @meets ),
            records => \@meets,
        },
    );
}

sub add_groups($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;

    IDS:
    foreach my $id ( @{ $args->{groups} } ) {
        $meet->find_or_create_related('meet_groups',
            { group_id => $id }
        );
    }
    $meet->update_groups_cache();

    $c->render( status  => 204, text => '' );
}

sub add_moderators($c) {
    $c->openapi->valid_input or return;
    my $args = $c->req->json;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;

    IDS:
    foreach my $user ( @{ $args->{users} } ) {

        $meet->find_or_create_related('moderators',
            {
                octid => $user->{id},
                name  => $user->{name},
            }
        );
    }
    $meet->update_moderators_cache();

    $c->render( status  => 204, text => '' );
}

sub delete_group($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;

    $meet->delete_related('meet_groups',
        { group_id => $c->stash->{group_id} }
    );
    $meet->update_groups_cache();

    $c->render( status  => 204, text => '' );
}

sub delete_moderator($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;

    $meet->delete_related('moderators',
        { octid => $c->stash->{user_id} }
    );
    $meet->update_moderators_cache();

    $c->render( status  => 204, text => '' );
}

sub delete($c) {
    $c->openapi->valid_input or return;

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    return $c->error(403, 'ACCESS_DENIED') if $meet->owner_id != $c->stash->{user}->id;

    $meet->update({ deleted => \'now()'});

    $c->render( status  => 204, text => '' );
}

sub meet($c) {

    my $meet = $c->schema->resultset('Meet')->find({
        id => $c->stash->{id}
    });

    my $user = $c->schema->resultset('User')->find({
        id => $c->current_user->{id}
    });

    return $c->error(404, 'NOT_FOUND') if ! $meet;
    # CHECK MEET ACCESS

    $c->stash->{is_editable} = ( $meet->owner_id == $user->id);
    $c->stash->{token} = $user->meet_token($meet, $c->config);
    $c->stash->{meet}  = $meet;

    $c->render('meet');
}

1;

__END__

    my $avatar = join ('',
       $c->config->{piratar},
       AVATAR_SIZE, '/',
       $user->username,
       '.jpg',
    );
    $c->stash->{token} = Mojo::JWT->new(
        secret => $c->config->{jitsi_secret},
        claims => {
            aud  => 'semeet',
            iss  => 'semeet',
            sub  => 'meet.pirati.cz',
            room => $meet->uuid,
#            moderator => $user{moderator} ? \1:\0,
            exp  => time + JITSI_TOKEN_LIFETIME,
            context => {
                user   => {
                    avatar => $avatar,
                    name   => $user->displayname,
#                    email  => $user{mail},
                }
            },
        }
    )->encode;
